package com.connor.jd.plm.dialog;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.ScrollPane;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.HashMap;
import java.util.Map;
import java.util.Map.Entry;

import javax.swing.BorderFactory;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import com.teamcenter.rac.aif.AbstractAIFApplication;
import com.teamcenter.rac.aif.AbstractAIFDialog;
import com.teamcenter.rac.aifrcp.AIFUtility;
import com.teamcenter.rac.kernel.TCClassificationService;
import com.teamcenter.rac.kernel.TCComponent;
import com.teamcenter.rac.kernel.TCComponentICO;
import com.teamcenter.rac.kernel.TCComponentItemRevision;
import com.teamcenter.rac.kernel.TCException;
import com.teamcenter.rac.kernel.TCProperty;
import com.teamcenter.rac.kernel.TCSession;
import com.teamcenter.rac.kernel.ics.ICSApplicationObject;
import com.teamcenter.rac.kernel.ics.ICSProperty;
import com.teamcenter.rac.kernel.ics.ICSPropertyDescription;
import com.teamcenter.rac.util.MessageBox;

public class EditClassificationDialog extends AbstractAIFDialog {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	private AbstractAIFApplication app;
	private TCSession session;
	private TCComponent target;
	private Map<String, JTextField> propMap;
	private TCClassificationService myService;
	final EditClassificationDialog DIALOG = this;

	public EditClassificationDialog(AbstractAIFApplication app) throws TCException {
		this.app = app;
		this.session = (TCSession) app.getSession();
		this.myService = session.getClassificationService();
		this.target = (TCComponent) AIFUtility.getCurrentApplication().getTargetComponent();
		if (target instanceof TCComponentItemRevision) {
			try {
				initUI();
			} catch (Exception e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		} else {
			MessageBox.post("ѡ汾", "", MessageBox.ERROR);
		}
	}

	private void initUI() {
		this.setTitle("޸ķ");
		this.setAutoRequestFocus(true);
		this.setResizable(true);

		JPanel rootPanel = new JPanel(new BorderLayout());
		rootPanel.setBorder(BorderFactory.createLineBorder(Color.black, 1));
		GridBagConstraints gbs = new GridBagConstraints();
		GridBagLayout gbl = new GridBagLayout();
		JPanel editPanel = new JPanel(gbl);
		gbs.anchor = GridBagConstraints.NORTHWEST;
		gbs.fill = GridBagConstraints.BOTH;
		gbs.insets = new Insets(10, 20, 5, 10);
		gbs.weighty = 0.2;
		gbs.gridheight = 1;

		Map<String, String> attr = null;
		propMap = new HashMap<String, JTextField>();
		TCComponentItemRevision rev = (TCComponentItemRevision) target;
		try {
			if (rev.getClassificationClass() != null || !"".equals(rev.getClassificationClass())) {
				try {
					attr = rev.getClassificationAttributes();

					for (Entry<String, String> entry : attr.entrySet()) {
						System.out.println(entry.getKey() + "===>" + entry.getValue());
					}
				} catch (TCException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}
			} else {
				MessageBox.post("ǰ汾δ", "", MessageBox.ERROR);
				return;
			}
		} catch (TCException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

		if (attr != null) {
			int num = 0;
			JLabel[] names = new JLabel[attr.size()];
			JTextField[] values = new JTextField[attr.size()];
			for (Entry<String, String> entry : attr.entrySet()) {
				gbs.gridwidth = 1;
				gbs.weightx = 0.4;
				names[num] = new JLabel(entry.getKey() + "");
				gbl.setConstraints(names[num], gbs);
				gbs.gridwidth = GridBagConstraints.REMAINDER;
				gbs.weightx = 2;
				values[num] = new JTextField(entry.getValue());
				propMap.put(entry.getKey(), values[num]);
				gbl.setConstraints(values[num], gbs);
				editPanel.add(names[num]);
				editPanel.add(values[num]);
				num++;
			}

			JPanel buttonPanel = new JPanel(new FlowLayout(FlowLayout.RIGHT));
			JButton cancelBtn = new JButton("ȡ");
			cancelBtn.addActionListener(cancel());
			JButton okBtn = new JButton("ȷ");
			okBtn.addActionListener(ok());
			buttonPanel.add(okBtn);
			buttonPanel.add(cancelBtn);

			ScrollPane scroll = new ScrollPane();
			scroll.add(editPanel);
			scroll.setPreferredSize(new Dimension(300, 300));
			rootPanel.add(scroll, BorderLayout.CENTER);
			rootPanel.add(buttonPanel, BorderLayout.SOUTH);
			this.setLayout(new BorderLayout());
			this.add(rootPanel);
			this.pack();
			this.centerToScreen();
			this.showDialog();
		}
	}

	private ActionListener ok() {
		return new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				// TODO Auto-generated method stub
				TCComponentItemRevision rev = (TCComponentItemRevision) target;
				saveClassification(rev);
				try {
					if (rev.getProperty("object_type").equals("JD2_GTBJRevision")) {
						TCProperty property = rev.getTCProperty("TC_Is_Represented_By");
						TCComponent[] comp = property.getReferenceValueArray();
						if (comp.length > 0) {
							TCComponentItemRevision tzrev = (TCComponentItemRevision) comp[0];
							saveClassification(tzrev);
						}
					}
				} catch (TCException e1) {
					// TODO Auto-generated catch block
					e1.printStackTrace();
				}
				DIALOG.disposeDialog();
			}
		};
	}

	private ActionListener cancel() {
		return new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				// TODO Auto-generated method stub
				DIALOG.disposeDialog();
			}
		};
	}

	private boolean saveClassification(TCComponentItemRevision rev) {

		try {
			ICSApplicationObject icsAppObj = myService.newICSApplicationObject("ICM");
			TCComponentICO[] icoS = rev.getClassificationObjects();
			String classID = rev.getClassificationClass();
			if (icoS == null || icoS.length == 0) {
				return false;
			}
			for (int i = 0; i < icoS.length; i++) {
				TCComponentICO ico = icoS[i];
				ICSPropertyDescription[] desc = ico.getICSPropertyDescriptors();
				ICSProperty[] oldICSProps = ico.getICSProperties(true);
				ICSProperty[] newICSProps = new ICSProperty[oldICSProps.length];
				for (int j = 0; j < oldICSProps.length; j++) {
					ICSProperty icsProp = oldICSProps[j];
					// icsProp.setValue("1111");
					newICSProps[j] = new ICSProperty(icsProp.getId(), "111111");
				}
				for (int j = 0; j < oldICSProps.length; j++) {
					newICSProps[j].setValue(propMap.get(desc[j].getName()).getText());
				}
				ico.delete();
				icsAppObj.create(myService.getTCComponentId(rev), myService.getTCComponentUid(rev));
				icsAppObj.setView(classID);

				icsAppObj.setProperties(newICSProps);
				icsAppObj.save();
			}
			rev.refresh();
		} catch (TCException e1) {
			// TODO Auto-generated catch block
			e1.printStackTrace();
		}
		return true;
	}
}
